import flask
import datetime
import json

app = flask.Flask(__name__)


def parse_event(line):
    data = json.loads(line)
    date = datetime.datetime.strptime(data["date"], "%d.%m.%Y")
    start_time = data["time"]
    sign = 1
    if '+' in start_time:
        start_time, timeshift = start_time.split('+')
        sign = -1
    elif '-' in start_time:
        start_time, timeshift = start_time.split('-')
        sign = 1
    else:
        timeshift = '00:00'
        sign = 1

    start_time = datetime.datetime.strptime(start_time, "%H:%M").time()
    timeshift = datetime.datetime.strptime(timeshift, "%H:%M").time()

    date = date.replace(minute=start_time.minute, hour=start_time.hour)
    start_date = date + sign * datetime.timedelta(minutes=timeshift.minute, hours=timeshift.hour) 
    
    end_date = start_date + datetime.timedelta(minutes=data["duration"])

    return start_date.date(), end_date.date()


def load_events(filename="input.txt"):
    events = set()
    with open(filename, "r") as f:
        for line in f:
            start_date, end_date = parse_event(line)
            events.add(start_date)
            for _ in range((end_date-start_date).days):
                start_date += datetime.timedelta(days=1)
                events.add(start_date)
        return events

ALL_EVENTS = load_events()


def find_free_day(target_date):
    if target_date not in ALL_EVENTS:
        return target_date
    for i in range(100000):
        d = target_date + datetime.timedelta(days=i)
        if d not in ALL_EVENTS:
            return d
        d = target_date + datetime.timedelta(days=-i)
        if d not in ALL_EVENTS:
            return d


@app.route("/find", methods=["GET"])
def find():
    target_date_str = flask.request.args.get("date")
    target_date = datetime.datetime.strptime(target_date_str, "%d.%m.%Y").date()
    date = find_free_day(target_date)
    return date.strftime("%d.%m.%Y")

if __name__ == "__main__":
    app.run(port=8080)
