import flask
import datetime

app = flask.Flask(__name__)

plants = {}


def date():
    return datetime.datetime.strptime(flask.request.args.get('date'), '%d.%m.%Y')


@app.route('/add/<plant>/<int:interval>/<int:maxdelay>/', methods=['GET'])
def add_plant(plant, interval, maxdelay):
    added_date = date()
    plants[plant] = {
        'interval': interval,
        'maxdelay': maxdelay,
        'added_date': added_date,
        'last_watered': added_date
    }
    return ''


@app.route('/task/', methods=['GET'])
def get_tasks():
    current_date = date()
    tasks = []

    for plant, data in list(plants.items()):
        last_watered = data['last_watered']
        next_watering_date = last_watered + datetime.timedelta(days=data['interval'])
        max_allowed_date = next_watering_date + datetime.timedelta(days=data['maxdelay'])

        if next_watering_date <= current_date <= max_allowed_date:
            tasks.append(plant)
        elif current_date > max_allowed_date:
            del plants[plant]

    return ','.join(tasks)


@app.route('/watering/<plant>/', methods=['GET'])
def water_plant(plant):
    current_date = date()
    if plant not in plants:
        return ''

    last_watered = plants[plant]['last_watered']
    next_watering_date = last_watered + datetime.timedelta(days=plants[plant]['interval'])
    max_allowed_date = next_watering_date + datetime.timedelta(days=plants[plant]['maxdelay'])
    if current_date < next_watering_date:
        del plants[plant]
    elif current_date > max_allowed_date:
        del plants[plant]
    else:
        plants[plant]['last_watered'] = current_date
    return ''

if __name__ == '__main__':
    app.run(host='0.0.0.0', port=8080)